<?php
namespace Service;

use Util\Period;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Service\Manager\Mongo\Manager;
use Util\Environment;
use Service\ColorPalette as ColorPaletteService;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Widget {

    /** @var \Service\ConnectionService $connectionService */
    protected $connectionService;
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $organizationManager;
    /** @var DataSource $dataSourceService */
    protected $dataSourceService;
    /** @var \Service\Structure $structureService */
    protected $structureService;
    /** @var \Service\Indicator $indicatorService */
    protected $indicatorService;
    /** @var \Service\ViewType $viewTypeService */
    protected $viewTypeService;
    /** @var \Service\Hierarchy $hierarchyService */
    protected $hierarchyService;
    /** @var \Service\Hierarchy $hierarchyTypeService */
    protected $hierarchyTypeService;
    /** @var \Service\ImportHistory */
    protected $importHistoryService;
    /** @var \Util\Environment $environment */
    protected $environment;
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $mongoManager;
    /** @var \Service\WidgetData $widgetDataService */
    protected $widgetDataService;
    /** @var \Service\ColorPaletteService $widgetDataService */
    protected $colorPaletteService;

    protected $dataSourceName = 'widget';
    protected $widgetFacts;

    const IS_DIMENSION = "isDimension";
    const IS_FACT = "isFact";
    const BAR_LINE_CHART_TYPE = "barLine";

    public function __construct(ConnectionService $connectionService, DataSource $dataSourceService, Structure $structure,
                                Indicator $indicatorService, Manager $organizationManager, ViewType $viewTypeService,
                                Hierarchy $hierarchyService, HierarchyType $hierarchyTypeService, ImportHistory $importHistoryService,
                                Environment $environment, Manager $mongoManager, $widgetDataService, ColorPaletteService $colorPaletteService) {
        $this->connectionService = $connectionService;
        $this->organizationManager = $organizationManager;
        $this->dataSourceService = $dataSourceService;
        $this->structureService = $structure;
        $this->indicatorService = $indicatorService;
        $this->viewTypeService = $viewTypeService;
        $this->hierarchyService = $hierarchyService;
        $this->hierarchyTypeService = $hierarchyTypeService;
        $this->importHistoryService = $importHistoryService;
        $this->environment = $environment;
        $this->mongoManager = $mongoManager;
        $this->widgetDataService = $widgetDataService;
        $this->colorPaletteService = $colorPaletteService;
    }

    public function findWidgetsByIndicator($indicatorId) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array( array(
            "columnName" => "indicator",
            "value" => $indicatorId,
            "operator" => "="
        )));

        $widgets = $this->organizationManager->findBy($filterCriteria)->getRows();
        $this->addDrillDownEvent($widgets);
        return $widgets;
    }

    public function findById($dataSource) {
        $operator = "=";
        if(is_array($dataSource)){
            $operator = "IN";
        }
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(array(
            "columnName"=>"_id",
            "value" => $dataSource,
            "operator" => $operator
        )));

        return $this->organizationManager->findBy($filterCriteria)->getRows();
    }

    public function findNext ($params) {
        $response = $this->organizationManager->findNext($this->dataSourceName, $params);
        return $response;
    }

    public function setLog($dataBefore, $id)
    {
        $dataAfter = $this->findById($id);
        $indicatorId = empty($dataAfter[0]['indicator']) ? $dataBefore[0]['indicator'] : $dataAfter[0]['indicator'];
        $label = empty($dataAfter[0]['label']) ? $dataBefore[0]['label'] : $dataAfter[0]['label'];
        Utilities::setLog($dataBefore, $dataAfter, $this->dataSourceName, $indicatorId, '_'.$id.'_'.$label);
    }

    protected function getWidgetLabelWithIndicatorName($widgetLabel, $indicatorName){
        return $widgetLabel." ------ [".$indicatorName."]";
    }

    public function findWithIndicatorNames($dataSet){
        $newDataSet = [];
        if(!empty($dataSet)){
            foreach($dataSet as $data){
                if(!empty($data["indicator"])){
                    $indicator = $this->indicatorService->findById($data["indicator"]);
                    if(!empty($indicator)){
                        $data["indicatorName"] = $indicator["name"];
                        $data["labelWithIndicator"] = $this->getWidgetLabelWithIndicatorName($data["label"],$indicator["name"]);
                        if(!empty($data["drillDownView"])){
                            $drillDown = $this->findById($data["drillDownView"]);
                            if(!empty($drillDown)){
                                $drillDownIndicator = $this->indicatorService->findById($drillDown[0]["indicator"]);
                                if(!empty($drillDownIndicator) && !empty($data["drillDownViewField"])){
                                    $data["drillDownViewField"] = $this->getWidgetLabelWithIndicatorName($data["drillDownViewField"],$drillDownIndicator["name"]);
                                }
                            }
                        }
                        $newDataSet[] = $data;
                    }
                }
            }
        }
        return $newDataSet;
    }

    public function getWidgetByIndicators($indicators) {
        $widgets = array();

        foreach ($indicators as $indicator) {
            $indicatorWidgets = $this->findWidgetsByIndicator($indicator['_id']);
            $widgets = array_merge($widgets, $indicatorWidgets);
        }

        return $widgets;
    }

    /**
     * @param $id
     * @return array
     * @throws \Exception
     */
    public function findWidgetById($id, $dashboardOnClick = null, $isViewPreview = false) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(
            array(
                "columnName" => "_id",
                "value"      => $id,
                "operator"   => "="
            )
        ));
        $widgets = $this->organizationManager->findBy($filterCriteria)->getRows();
        if($dashboardOnClick === "drillDown" || $dashboardOnClick === "drillDownPre" || $dashboardOnClick === "drillDownPos" || !$dashboardOnClick){
            $this->addDrillDownEvent($widgets,$dashboardOnClick);
        }else if(!$isViewPreview){
            if($dashboardOnClick === "filter" || $dashboardOnClick === "filterWithPeriod"){
                $this->addFilterEvent($widgets,$dashboardOnClick,$dashboardOnClick === "filterWithPeriod");
            }else if($dashboardOnClick === "filterPre"){
                $this->addFilterPreEvent($widgets,$dashboardOnClick);
            }else if($dashboardOnClick === "filterPos"){
                $this->addFilterPosEvent($widgets,$dashboardOnClick);
            }
        }

        if(count($widgets) > 0) {
            $widget = reset($widgets);
            return $widget;
        }

        throw new \Exception(Exception::buildErrorJSON(
            "INDICATOR_NOT_FOUND",
            array("id" => $id)
        ), 1);
    }

    public function getManyWidgetsById(array $ids) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(
            array(
                "columnName" => "_id",
                "value"      => $ids,
                "operator"   => FilterCriteria::IN
            )
        ));

        return $this->organizationManager->findBy($filterCriteria)->getRows();
    }

    protected function addDrillDownEvent(array &$widgets,$dashboardOnClick = "") {
        $event = array(
            "name" => "WidgetAfterMoveRow",
            "code" => "DrillDownController.openNextIndicatorView(args.owner, args.row, '".$dashboardOnClick."');"
        );
        return $this->addEventOnWidgets($widgets, $event,$dashboardOnClick);
    }

    protected function addFilterEvent(array &$widgets,$dashboardOnClick = null,$filterWithPeriod=false) {
        $filterWithPeriod=$filterWithPeriod ? "true" : "false";
        $event = array(
            "name" => "WidgetAfterMoveRow",
            "code" => "ViewController.filterDashboardByWidget(args.owner, args.row,".$filterWithPeriod.");"
        );

        return $this->addEventOnWidgets($widgets, $event,$dashboardOnClick);
    }

    protected function addFilterPreEvent(array &$widgets,$dashboardOnClick = null) {
        $event = array(
            "name" => "WidgetAfterMoveRow",
            "code" => "ViewController.filterPreDashboardByWidget(args.owner, args.row);"
        );

        return $this->addEventOnWidgets($widgets, $event,$dashboardOnClick);
    }

    protected function addFilterPosEvent(array &$widgets,$dashboardOnClick = null) {
        $event = array(
            "name" => "WidgetAfterMoveRow",
            "code" => "ViewController.filterPosDashboardByWidget(args.owner, args.row);"
        );

        return $this->addEventOnWidgets($widgets, $event,$dashboardOnClick);
    }

    protected function getEventFunctionName($event){
        return strtok($event["code"],"(");
    }

    protected function addEventOnWidgets(array &$widgets, $event,$dashboardOnClick = null)
    {
        foreach ($widgets as &$widget) {
            if (!isset($widget['events'])) {
                $widget['events'] = array();
            }
            if($widget["template"] == "widget/grid.html" && $event["name"] == "WidgetAfterMoveRow" && $dashboardOnClick){
                $widget['events'][] = array(
                    "name" => "WidgetOnDashboardClick",
                    "code" => "DashboardController.dashboardGridOnClick(args.owner, args.row, ".$this->getEventFunctionName($event).");"
                );
            } else {
                $widget['events'][] = $event;
            }
        }

        return $widgets;
    }

    protected function getFactMask($facts) {

        $return = null;
        if(!empty($facts)) {
            foreach ($facts as $fact) {
                if(isset($fact['maskField']) && !empty($fact['maskField']) && !$return) {
                    $return = $fact['maskField'];
                }
            }
        }
        return $return;
    }

    protected function addLabelOfColumn(&$view, array $facts, array $labels) {
        if(!empty($labels)) {
            $factsMap = array();
            foreach ($facts as $fact) {
                $factsMap[$fact['name']] = $fact;
            }
            $this->setLabelsOnChart($view, $factsMap, $labels);
        }
    }

    protected function setLabelsOnChart(&$view, $factsMap, $labels) {
        foreach ($labels as $property => $values) {
            $view[$property] = array();
            foreach ($values as $value) {
                $value = is_array($value) ? $value[0] : $value;
                $view[$property][] = (isset($factsMap[$value]) ? $factsMap[$value]['label'] : $value);
            }
        }
    }

    protected function setCustomDimension(&$view, $viewType, $customDimension = null) {
        if(isset($viewType['dimensionProperty']) && is_array($viewType['dimensionProperty'])) {
            $viewDimensionProperty = $viewType['dimensionProperty']['name'];
            $view['dimensionProperty'] = $viewDimensionProperty;
            if($customDimension) {
                $view[$viewDimensionProperty] = $customDimension;
            }
        }
    }

    protected function hasMultiplePeriod($period) {
        return is_array($period) && !empty($period) && Utilities::isNumericArray($period);
    }

    protected function hasProperty($arr, $property) {
        return isset($arr[$property]);
    }

    protected function setMultiplePeriodsOnChart(&$view, $period, $propertyName, $factsNames, &$labels, $dataSource) {
        $suffixedNames = array();
        $masks = array();
        foreach($period as $singlePeriod) {
            if ($singlePeriod != null) {
                $labelSuffix = null;
                if (is_string($singlePeriod)) {
                    $periodObj = $this->getPeriodObjFromCode($singlePeriod);
                    if (isset($periodObj['labelOnView']) || isset($periodObj['name'])) {
                        $labelSuffix = $periodObj['labelOnView'] ?: $periodObj['name'];
                    }
                }

                $period = $this->decodePeriod($singlePeriod);
                $suffix = implode("_", Period::formatInterval(array($period['start'], $period['end'])));

                if (!$labelSuffix) {
                    $labelSuffix = implode(" to ", Period::formatInterval(array($period['start'], $period['end'])));
                }

                $facts = $this->widgetDataService->getDataSourceColumns($dataSource, $factsNames, $view);

                foreach ($facts as $fact) {
                    $suffixedName = "{$fact['name']}_{$suffix}";
                    $suffixedNames[] = $suffixedName;
                    $view["factsNames"][$suffixedName] = $fact["name"];
                    $labels[] = "{$fact['label']} - {$labelSuffix}";
                    $masks[] = isset($fact['mask']) ? $fact['mask'] : null;
                }
            }else{
                $facts = $this->widgetDataService->getDataSourceColumns($dataSource, $factsNames, $view);
                foreach ($facts as $fact) {
                    $suffixedNames[] = "{$fact['name']}";
                    $labels[] = "{$fact['label']} - Todo o Período";
                }
            }
        }

        if(isset($fact)){
            $view['chartSerieField'] = $fact;
        }
        $view[$propertyName] = $suffixedNames;
        $view['masks'] = $masks;
    }

    public function getPeriodObjFromCode($code)
    {
        if (preg_match('/^[0-9a-f]{24}$/i', $code) === 1) {
            /** @var $periodsService \Service\PeriodsService */
            $periodsService = InstanceManager::getInstance()->getService('\Service\PeriodsService');
            $periodObj = $periodsService->findById($code);
            if (empty($periodObj)) {
                $periodObj =  array(
                    'custom' => true,
                    'description'=> "",
                    'name'=> "Hoje",
                    'periodsCode'=> "today",
                    "base" => true
                );
            }
        }
        if (empty($periodObj)) {
            $filterCriteria = new FilterCriteria('periods');
            $filterCriteria->addCondition('periodsCode', '=', $code);
            $rows = $this->organizationManager->findBy($filterCriteria)->getRows();
            if (empty($rows)) {
                $rows = $this->mongoManager->findBy($filterCriteria)->getRows();
                if(!empty($rows) && !empty($rows[0])){
                    $rows[0]["base"] = true;
                }
            }
            $periodObj = reset($rows);
        }
        $periodObj['labelOnView'] = isset($periodObj['labelOnView']) ? $periodObj['labelOnView'] : null;

        return $periodObj;
    }

    protected function decodePeriod($period) {
        $periodToReturn = array();
        if(is_string($period)) {
            $periodName = $period;
            list($periodToReturn['start'], $periodToReturn['end']) = Period::decode($periodName);
        } else if(isset($period['start']) && isset($period['end'])) {
            list($periodToReturn['start'], $periodToReturn['end']) = Period::convertStringToMoment($period['start'], $period['end']);
        } else {
            $periodToReturn = $period;
        }
        return $periodToReturn;
    }

    protected function setViewColumnsWithoutCustomFacts(&$view, $viewType, $period, &$facts, &$labels, $dataSource) {
        $facts = array();
        if(isset($viewType['factProperties']) && is_array($viewType['factProperties'])) {
            foreach ($viewType['factProperties'] as $property) {
                $this->setColumnsWithoutCustomFacts($property, $view, $period, $facts, $labels, $dataSource);
            }
        } else if(isset($viewType['factProperty']) && is_array($viewType['factProperty'])) {
            $this->setColumnsWithoutCustomFacts($viewType['factProperty'], $view, $period, $facts, $labels, $dataSource);
        }

        $this->widgetFacts = $facts;
    }

    protected function setColumnsWithoutCustomFacts($property, &$view,&$period, &$facts, &$labels, &$dataSource) {
        $propertyName = $property['name'];

        if(isset($property["convertToNumber"]) && $property["convertToNumber"]) {
            $view["convertToNumber"] = $property["convertToNumber"];
        }

        $view['factProperties'][] = $propertyName;

        if($this->hasProperty($view, $propertyName) && is_array($view[$propertyName])) {
            $factsNames = $view[$propertyName];
            $customLabels = $labels;

            if($this->hasMultiplePeriod($period)) {
                $this->setMultiplePeriodsOnChart($view, $period, $propertyName, $factsNames, $customLabels, $dataSource);
            }

            if(empty($customLabels)) {
                $customLabels = $factsNames;
            }
            $facts = array_merge($facts, $factsNames);
            if(!empty($property['outLabelProperty'])) {
                $labels[$property['outLabelProperty']] = $customLabels;
            }
        }
    }

    protected function setViewColumnsWithCustomFacts(&$view, $viewType, $period, &$facts, &$labels, $dataSource) {
        if(isset($viewType['factProperties']) && is_array($viewType['factProperties'])) {
            foreach ($viewType['factProperties'] as $property) {
                $this->setColumnsWithCustomFacts($property, $view, $period, $facts, $labels, $dataSource);
            }
        } else if(isset($viewType['factProperty']) && is_array($viewType['factProperty'])) {
            $this->setColumnsWithCustomFacts($viewType['factProperty'], $view, $period, $facts, $labels, $dataSource);
        }

        $this->widgetFacts = $facts;
    }

    protected function setColumnsWithCustomFacts($property, &$view, &$period, &$facts, &$labels, &$dataSource) {
        $propertyName = $property['name'];
        if(isset($property["convertToNumber"]) && $property["convertToNumber"]) {
            $view["convertToNumber"] = $property["convertToNumber"];
        }
        $view['factProperties'][] = $propertyName;
        $customLabels = $labels;
        if($this->hasMultiplePeriod($period)){
            $this->setMultiplePeriodsOnChart($view, $period, $propertyName, $facts, $customLabels, $dataSource);
        } else {
            $view[$propertyName] =  $facts;
        }

        if(empty($customLabels)) {
            $customLabels = $facts;
        }

        if(!empty($property['outLabelProperty'])) {
            $labels[$property['outLabelProperty']] = $customLabels;
        }
    }

    protected function setCustomFactsAndMask(&$view, $viewType, $dataSource,
                                             $customFacts = null, $lang = null,
                                             $period = array(), $valuePrecision = null) {
        if(isset($viewType['factProperties']) && is_array($viewType['factProperties'])) {
            $view['factProperties'] = array();
            $facts = $customFacts;
            $labels = array();
            if(empty($facts)) {
                $this->setViewColumnsWithoutCustomFacts($view, $viewType, $period, $facts, $labels, $dataSource);
            } else {
                $this->setViewColumnsWithCustomFacts($view, $viewType, $period, $facts, $labels, $dataSource);
            }

            if(!empty($facts)) {
                $facts = $this->widgetDataService->getDataSourceColumns($dataSource, $facts, $view);

                $this->addLabelOfColumn($view, $facts, $labels);

                $maskType = $this->getFactMask($facts);

                switch ($view['chartType']) {
                    case "bar":
                    case "line":
                    case "area":
                    case "mixed":
                        $view['yAxisMask'] = $maskType;
                        break;
                    case "gauge":
                    case "pie":
                        $view['hintFormat'] = $maskType;
                        break;
                    default:
                        $view['mask'] = array(
                            "type" => $maskType,
                            "params" => array(
                                "inputLang" => $lang,
                            )
                        );
                }
            }
        } else if(isset($viewType['factProperty']) && is_array($viewType['factProperty'])) {
            $view['factProperties'] = array();
            $facts = $customFacts;
            $labels = array();

            if(empty($facts)) {
                $this->setViewColumnsWithoutCustomFacts($view, $viewType, $period, $facts, $labels, $dataSource);
            } else {
                $this->setViewColumnsWithCustomFacts($view, $viewType, $period, $facts, $labels, $dataSource);
            }

            if(!empty($facts)) {
                $facts = $this->widgetDataService->getDataSourceColumns($dataSource, $facts, $view);

                $this->addLabelOfColumn($view, $facts, $labels);
                $maskType = $this->getFactMask($facts);

                $serieTypeProperty = $view['serieType'] == "bar" ? "serieYAxisFormat" : "serieValueFormat";

                if(isset($view[$serieTypeProperty]) && $view[$serieTypeProperty] == "float" && $valuePrecision !== null){
                    $view[$serieTypeProperty] = "value.formatFloat($valuePrecision)";
                } else{
                    $view[$serieTypeProperty] = isset($view[$serieTypeProperty]) ? $view[$serieTypeProperty] : $maskType;
                }

            }
        }
    }

    protected function createColumnsFromNameLabel($names, $labels, $masks = array()) {
        $columnsToReturn = array();
        foreach($names as $index => $name) {
            $columnsToReturn[] = array(
                "name" => $name,
                "label" => isset($labels[$index]) ? $labels[$index] : $name,
                "mask" => isset($masks[$index]) ? $masks[$index] : null
            );
        }
        return $columnsToReturn;
    }

    protected function convertPropertiesToColumns(&$view, $viewType, $dataSource) {
        if(isset($viewType['convertToColumns']) && !empty($viewType['convertToColumns'])) {
            foreach ($viewType['convertToColumns'] as $property) {
                $columns = $this->widgetDataService->getDataSourceColumns($dataSource, $view[$property['name']], $view);
                if (empty($columns)) {
                    foreach ($viewType['factProperties'] as $factProperty) {
                        $columns = $this->createColumnsFromNameLabel($view[$factProperty['name']],
                            isset($factProperty['outLabelProperty']) ? $view[$factProperty['outLabelProperty']] : $view[$factProperty['name']],
                            isset($view['masks']) ? $view['masks'] : array()
                            );
                    }
                }
                foreach ($columns as &$column) {
                    $column['showOnList'] = true;
                }

                $outputProperty = isset($property['outputProperty']) ? $property['outputProperty'] : $property['name'];
                if(!isset($view[$outputProperty]) || !is_array($view[$outputProperty])) {
                    $view[$outputProperty] = array();
                }
                $view[$outputProperty] = array_merge($view[$outputProperty], $columns);

                foreach ($view[$outputProperty] as &$field) {
                    if(isset($field['mask'])) {
                        if ($field['mask'] == "") {
                            $field['mask'] = null;
                        } else {
                            $field['align'] = "right";
                            if (!is_array($field['mask'])) {
                                $field['mask'] = array(
                                    "type" => $field['mask'],
                                    "params" => (object)array()
                                );
                            }
                        }
                    }
                }
            }
        }
    }

    public function makeWidget($viewId, $indicatorId = null, $dimension = null, $facts = array(), $lang = null,
                                $order = null, $period = null, $color = null, $dashboardOnClick = null, $isViewPreview = false) {

        $view = $this->findWidgetById($viewId, $dashboardOnClick, $isViewPreview);
        list($view) = Utilities::setPeriod(array($view));

        if(isset($view['mobile'])){
            $view['mobile']= [];
        }
        $view['actionOnClick'] = $this->getActionOnclick($dashboardOnClick);

        $valuePrecision = isset($view["valuePrecision"]) ? $view["valuePrecision"] : null;
        $view['id'] = $viewId;
        $viewType = $this->viewTypeService->findById($view['viewTypeId']);

        if(empty($indicatorId)) {
            $indicatorId = $view['indicator'];
        }
        $indicator = $this->indicatorService->findById($indicatorId);
        $dataSource = $this->dataSourceService->findByIndicator($indicatorId);

        $period = empty($period) && $this->hasProperty($view, 'period') ? $view['period'] : $period;

        if($view['chartType'] == 'periodsList' && $view['viewType'] == 'periodsList'){
            $view['periodsListFact'] = $view['periodsFact'];
        }
        $this->setCustomFactsAndMask($view, $viewType, $dataSource, $facts, $lang, $period, $valuePrecision);
        if(isset($view['periodsListFact'])){
            $view['periodsFact'] = $view['periodsListFact'];
        }
        $this->setCustomDimension($view, $viewType, $dimension);
        $this->convertPropertiesToColumns($view, $viewType, $dataSource);
        $this->verifyZmartTotalized($view);
        $this->addTotalizedPropertiesToMultiplePeriodFields($view,$period);


        if(isset($viewType['template'])) {
            $view['template'] = $viewType['template'];
        }
        $view['dataSourceId'] = $dataSource['_id'];
        $view['name'] = isset($view['label']) ? $view['label'] : $indicator['name'];
        $view['floatingControl'] = array("configAction" => false, "customizationAction" => array("layout" => true, "filter"=>false, "view"=>false));
        if($view['template'] == "widget/grid.html"){
            $view['floatingControl']['xlsAction'] = array("runAt" =>"server");
            $view['floatingControl']['pdfAction'] = array("runAt" =>"server");
            $view['floatingControl']['csvAction'] = true;
            $view['hasPagination'] = false;
            $view['searchWithPagination'] = false;
        }

        $view['events'][] = array(
            "name" => "BeforeGenerateReport",
            "code" => "ViewController.beforeGenerateReportByWidget(args);"
        );

        $view = $this->setCrossTabProperty($view);

        $view['order'] = $order;
        $view['color'] = $color;
        $view['reportShowOnlyVisibleFields'] = true;

        $serie = array();
        foreach($view as $property => $propertyValue) {
            if(substr($property, 0, 5) === "serie") {
                if(
                    isset($viewType['factProperty']) &&
                    isset($viewType['factProperty']['name']) &&
                    $viewType['factProperty']['name'] === $property &&
                    isset($viewType['factProperty']['convertToNumber']) &&
                    $viewType['factProperty']['convertToNumber']
                ) {
                    $serie[$property] = 0;
                } else {
                    $serie[$property] = $propertyValue;
                }
            }
        }

        if($view['viewType'] !== self::BAR_LINE_CHART_TYPE){
            $view['chartSeries'] = array($serie);
        }

        $view = $this->fixBugReport($view);
        $view = $this->insertIdIntoBuildedWidgetFields($view);
        return array($view, $viewType);
    }

    protected function addTotalizedPropertiesToMultiplePeriodFields(&$view,$period){
        if(!empty($view["zmartTotalized"]) && $view["chartType"] === "grid" && $this->hasMultiplePeriod($period)){
            foreach($view["yKeys"] as $fact){
                foreach($view["fields"] as &$field){
                    if($field["name"] == $fact){
                        foreach($view["columns"] as $column){
                            if($column["name"] == $view["factsNames"][$fact]){
                                $originalColumn = $column;
                                break;
                            }
                        }
                        $field = array_merge($originalColumn,$field);
                        $field["expression"] = $field["function"];
                    }
                }
            }
        }
    }

    public function getActionOnclick($dashboardOnClick)
    {
        if($dashboardOnClick === "drillDown" || !$dashboardOnClick){
            return 'drillDown';
        }else if($dashboardOnClick === "filter"){
            return 'filter';
        }else if($dashboardOnClick === "filterWithPeriod"){
            return 'filterWithPeriod';
        }else if($dashboardOnClick === "filterPre"){
            return 'filterPre';
        }else if($dashboardOnClick === "filterPos"){
            return 'filterPos';
        }else if($dashboardOnClick === "drillDownPre"){
            return 'drillDownPre';
        }else if($dashboardOnClick === "drillDownPos"){
            return 'drillDownPos';
        }
    }

    protected function insertIdIntoBuildedWidgetFields($widget) {
        if(isset($widget['fields'])){
            foreach($widget['fields'] as &$field){
                $field['id'] = $field['name'] . $widget['id'];
            }
        }
        return  $widget;
    }

    protected function getDataSourceConnectionLanguage($viewId, $isViewPreview = false) {
        try{
            $view = $this->findWidgetById($viewId,null,$isViewPreview);
            $dataSource = $this->dataSourceService->findByIndicator($view['indicator']);
            $connectionId = isset($dataSource['connection']) ? $dataSource['connection'] : null;
            $connection = $this->connectionService->getConnectionParams($connectionId);
            if(isset($connection['language'])) {
                return $connection['language'];
            }
        }catch(\Exception $e){

        }
        return null;
    }

    private function setColors($widget, $serieFields = "serieFields", $chartIndex = 0){
        if(empty($widget["chartSeries"][$chartIndex]["serieColors"])){
            $widget["chartSeries"][$chartIndex]["serieColors"] = [];
        }
        if(!empty($widget[$serieFields])){
            $defaultColors = $this->colorPaletteService->find(new FilterCriteria('color'),false,true);
            foreach($defaultColors as $i => $color){
                $defaultColors[$i] = $color["color"];
            }
            $defaultColorsCount = count($defaultColors);
            $oldColorsCount = count($widget["chartSeries"][$chartIndex]["serieColors"]);
            foreach($widget["chartSeries"][$chartIndex]["serieFields"] as $i => $name){
                if($i >= $oldColorsCount){
                    if(!empty($widget["parameters"]["period"][$i])){
                        $name = str_replace("_".$widget["parameters"]["period"][$i]["start"],"",str_replace("_".$widget["parameters"]["period"][$i]["end"],"",$name));
                    }
                    $col = array_values(array_filter(
                        $widget["columns"],
                        function ($e) use ($name) {
                            return $e["name"] == $name;
                        }
                    ));
                    if(!empty($col[0]["color"][0])){
                        array_push($widget["chartSeries"][$chartIndex]["serieColors"],$col[0]["color"][0]);
                    }else{
                        array_push($widget["chartSeries"][$chartIndex]["serieColors"],$defaultColors[$i % $defaultColorsCount]);
                    }
                }
            }
        }
        return $widget["chartSeries"][$chartIndex]["serieColors"];
    }

    public function buildWidgetToRender($viewId, $structureId = null, $period = null, $dimension = null,
                                        $facts = array(), $orderBy = array(), $limitRows = null,
                                        $drillDownFilters = null, $order = null, $color = null,
                                        $dashboardOnClick = null, $serieColors = null,
                                        $userStructureHierarchy = null, $dimensionX = null,
                                        $dimensionY= null, $structureName = null, $isViewPreview = false,
                                        $hierarchyList = null, $mockPeriod = null) {
        $this->organizationManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        if($isViewPreview){
            $importDatabase = Utilities::getImportDefaultDatabase();
            $this->organizationManager->createDatabase($importDatabase);
        }
        $language = $this->getDataSourceConnectionLanguage($viewId,$isViewPreview);
        list($widget, $view) = $this->makeWidget($viewId, null, $dimension, $facts, $language, $order, $period,
            $color, $dashboardOnClick, $isViewPreview);

        $widget['dataSource'] = array(
            "rest" => true,
            "localStorage" => false,
            "name" => "/view/fetchWidgetData",
            "lazyLoad" => true,
            "timeout"=> 120000,
            "data" => array()
        );

        if($isViewPreview && !empty($mockPeriod)){
            $widget["mockPeriod"] = $mockPeriod;
        }

        $widget["parameters"] = $this->getFilterParameters($widget, $view, $period, $structureId, $dimension, $facts,
            $limitRows, $drillDownFilters, $orderBy, $userStructureHierarchy, $dimensionX , $dimensionY, $structureName, $hierarchyList);
        if($serieColors && isset($widget["chartSeries"][0]["serieLabels"]) && count($widget["chartSeries"][0]["serieLabels"]) === 1) {
            $widget["serieColors"] = $serieColors;
            $widget["chartSeries"][0]["serieColors"] = $serieColors;
        }

        foreach($widget["chartSeries"] as $i => $chart){
            if ($widget['chartType'] === 'bar' && $chart['serieType'] == "bar"){
                $widget["chartSeries"][$i]["serieYAxisPosition"] = 'left';
            }
            if($chart['serieType'] == "bar" && isset($widget['serieFieldsBar'])){
                $widget["chartSeries"][$i]["serieColors"] = $this->setColors($widget,'serieFieldsBar', $i);
            }else if($chart['serieType'] == "line" && isset($widget['serieFieldsLine'])){
                $widget["chartSeries"][$i]["serieColors"] = $this->setColors($widget,'serieFieldsLine', $i);
            }else{
                $widget["chartSeries"][$i]["serieColors"] = $this->setColors($widget);
            }
        }

        if(isset($widget["parameters"]["orderByName"])){
            $orderByName = $widget["parameters"]["orderByName"];
        }else{
            $orderByName = "";
        }
        if(isset($widget["parameters"]["orderByDirection"])){
            $orderByDirection = $widget["parameters"]["orderByDirection"];
        }else{
            $orderByDirection = "";
        }

        $widget['dataSourceFilter'] = array(
            array("name" =>
                array(
                    array("name" => $orderByName, "direction" => $orderByDirection)
                ),
                "value" => null,
                "operator" => "ORDER_BY"
            )
        );

        if($widget['template'] === 'zmart-components#compare-periods.html' && !empty($widget['parameters']['drillDownFilters'])) {
            $widget['period'] = $this->setPeriodFormat($widget['period']);
        }
        return $widget;
    }
    public function setPeriodFormatMock($period,$indicatorId)
    {
        if($period){
            $dateBase = '';
            $filterCriteria = new FilterCriteria('indicator', array(array(
                "columnName"=>"_id",
                "value" => $indicatorId,
                "operator" => "="
            )));
            $indicators = $this->organizationManager->findBy($filterCriteria)->getRows();
            if(!empty($indicators[0]['dataSource'])){
                $filterCriteria = new FilterCriteria('zmartLog', array(
                    array(
                        "columnName"=>"collectionId",
                        "value" => $indicators[0]['dataSource'],
                        "operator" => "="
                    ),
                    array(
                        "columnName"=>"collectionName",
                        "value" => 'dataSource',
                        "operator" => "="
                    ),
                    array(
                        "columnName"=>"VALUE_BEFORE",
                        "value" => 'Create',
                        "operator" => "="
                    )
                ));
                $filterCriteria->setPageSize(1);
                $filterCriteria->setPage(1);
                $filterCriteria->addOrderBy('DATA_UPDATE');
                $zmartLog = $this->organizationManager->findBy($filterCriteria)->getRows();
                if(!empty($zmartLog) && !empty($zmartLog[0]) && $zmartLog[0]['DATA_UPDATE']){
                    $dateBase = $zmartLog[0]['DATA_UPDATE']->format('Y-m-d');
                }
            }
            if($this->hasMultiplePeriod($period)) {
                $periods = array();
                foreach ($period as $singlePeriod) {
                    $periods[] = $this->decodeFormattedPeriod($singlePeriod,$dateBase);
                }
                $period = $periods;
            } else {
                $period = $this->decodeFormattedPeriod($period,$dateBase);
                $parameters['period_start'] = $period["start"];
                $parameters['period_end'] = $period["end"];
            }
        }
        return $period;
    }

    public function setPeriodFormat($period)
    {
        if($period){
            if($this->hasMultiplePeriod($period)) {
                $periods = array();
                foreach ($period as $singlePeriod) {
                    $periods[] = $this->decodeFormattedPeriod($singlePeriod);
                }
                $period = $periods;
            } else {
                $period = $this->decodeFormattedPeriod($period);
                $parameters['period_start'] = $period["start"];
                $parameters['period_end'] = $period["end"];
            }
        }
        return $period;
    }

    public function getWidgetLastImportDate(&$widget){
        $widget["lastImportation"] = $this->importHistoryService->getLastImportationDate($widget['dataSourceId']);
    }

    public function zeedhiToNext($widget){
        $widgetNext = array();
        if($widget["template"]  == "zh-charts#widget/zh-chart.html"){
            if($widget["chartType"] == "column"){

            }

        }
        $filter = array(
            "drillDownFilters" => [],
            "structureId" => [$widget['parameters']['structureId']],
            "structureName" => "",
            "viewId" => $widget['_id'],
            "serieColors" => 0,
            "period" => $widget['parameters']['period'],
            "zmartTotalized" => false
        );
        /** @var $widgetData \Service\WidgetData */
        $widgetData = InstanceManager::getInstance()->getService('\Service\WidgetData');
        $data = $widgetData->fetchWidgetData($widget['_id'], $widget['parameters']['structureId'], null, $widget['parameters']['period'],
            $widget['parameters']['facts'], $widget['parameters']['dimensionName'],array(),null,null,$widget['parameters']['drillDownFilters']);

        $categoria = array();
        $series = array();
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
            foreach ($widget['serieFields'] as $index => $fact) {
                if(empty($series[$index])){
                    $series[$index] = array(
                        "name" => $widget['serieLabels'][$index] ,
                        "field" => $fact ,
                        "data" => array()
                    );
                }
                $series[$index]["data"][] = $row[$fact];
            }
        }
        $legendPosition = "top";
        if($widget["chartLegendPosition"]){
            $legendPosition = explode(" ",$widget["chartLegendPosition"])[0];
        }
        $serieColors = array();
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            $serieColors = $widget["chartSeries"][0]["serieColors"];
        }
        $serieStacked = false;
        if(!empty($widget['serieStacked'])){
            $serieStacked = $widget['serieStacked'];
        }

        $title = [];
        if(!empty($widget['serieYAxisLabel'])){
            $title['text'] = $widget['serieYAxisLabel'];
        }

        $opposite = false;
        if(!empty($widget['serieYAxisPosition'])){
            $opposite = $widget['serieYAxisPosition'];
        }
        $dynamicWidth = count($data) * 1000;
        $widgetNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => "579106a111e8f41bb00caff6",
            "component" => "ZdApexChart",
            "chartType" => $widget['serieType'],
            "cssStyle" => "overflow-y: hidden; overflow-x: auto;",
            "legend" => [
                "position" => 'top',
            ],
            "height" =>"100%",
            //"width" => $dynamicWidth,
            "options" => [
                "colors" => $serieColors,
                "drillDownView" => $widget['drillDownView'],
                "plotOptions" => [
                    "bar"=> [
                        "horizontal" => !empty($widget['chartInvertedAxes']),
                        "dataLabels" => [
                            "position" => !empty($widget['chartInvertedAxes']) ? 'left' : 'top'
                        ]
                    ]
                ],
                "dataLabels" => [
                    "enabled" => true,
                    "position" => "top",
                    "style" => [
                       // "colors" => $serieStacked ? [] : $serieColors
                        "colors" => []
                    ],
                    "offsetY" => -16,
                ],
                "chart" => [
                    "id" => $widget['id'],
                    "stacked" => $serieStacked,
                    "toolbar" => [
                        "show" => true,
                        "tools" => [
                            "download" => true,
                            "selection" => false,
                            "zoom" => false,
                            "zoomin" => false,
                            "zoomout" => false,
                            "pan" => false,
                            "reset" => false,
                            "customIcons" => []
                        ],
                    ],
                ],
                "legend" => [
                    "position" => $legendPosition
                ],
                "zoom" > [
                    "enabled" => true
                ],
                "title" => [
                    "text" => $widget['name'],
                    "style" => [
                        "fontSize" => "16px"
                    ]
                ],
                "xaxis" => [
                    "categories" => $categoria,
                    "field" => $widget['chartXAxisField'],
                    "labels" => [
                        "rotate" =>  -20,
                        "show" => true,
                    ],
                    "tickPlacement" => "between"
                ],
                "yaxis" => [
                    "mask" => $widget['chartSeries'][0]['serieYAxisFormat'],
                    "align" => 'right',
                    "opposite" => $opposite,
                    "dataLabels" => [
                    ],
                    "title" => $title,
                    "labels" => [
                        "show" => true,
                    ]
                ]
            ],
            "series" => $series

        ];
//        $widgetNext = [
//            "name" => "apexChart",
//            "component" => "ZdApexChart",
//            "chartType" => "bar",
//            "grid" => [
//                "sm" => "8"
//            ],
//            "options" => [
//                "chart" => [
//                    "id" => "bar-chart"
//                ],
//                "title" => [
//                    "text" => "Basic chart",
//                    "style" => [
//                        "fontSize" => "16px"
//                    ]
//                ],
//                "xaxis" => [
//                    "categories" => [
//                        'tytyu',
//                        1992,
//                        1993,
//                        1994,
//                        1995,
//                        1996,
//                        1997,
//                        1998
//                    ]
//                ]
//            ],
//            "series" => [
//                [
//                    "name" => "series-1",
//                    "data" => [
//                        30,
//                        40,
//                        45,
//                        50,
//                        49,
//                        60,
//                        70,
//                        91
//                    ]
//                ]
//            ]
//
//        ];


        return $widgetNext;
    }

    protected function getFilterParameters($widget, $view, $period = array(), $structureId = null, $dimension = null,
                                           $facts = array(), $limitRows = null, $drillDownFilters = null, $orderBy,
                                           $userStructureHierarchy = null, $dimensionX = null , $dimensionY= null, $structureName = null, $hierarchyList = null) {
        $parameters = array();
        if(isset($widget['indicator'])){
            $this->getIndicatorParameters($widget['indicator'], $parameters);
        }

        $this->getDimensionParameters($widget, $dimension, $parameters, $view);

        if(empty($facts)) {
            $facts = $this->getFactsParameter($widget);
        }
        $parameters['facts'] = $facts;

        $this->getOrderByParameters($widget, $facts, $dimension, $parameters, $orderBy);
        $this->getPeriodParameters($widget, $parameters, $period);
        $this->getStructureParameters($widget['indicator'], $parameters, $structureId, $userStructureHierarchy, $structureName, $hierarchyList);

        if($limitRows) {
            $parameters["limitRows"] = $limitRows;
        }

        $parameters['drillDownFilters'] = $drillDownFilters;

        if($widget['serieType'] == 'crosstab'){
            $parameters['dimensionY'] = $dimensionY ? $dimensionY : $widget['yAxisProperty'];

            $parameters['dimensionX'] = $dimensionX ? $dimensionX : $widget['xAxisProperty'];

            if(!is_array($parameters['dimensionY'])){
                $parameters['dimensionY'] = array($parameters['dimensionY']);
            }
            if(is_array($parameters['dimensionX'])){
                $parameters['dimensionX'] = $parameters['dimensionX'][0];
            }

            if($widget['xKeysX_select']){
                $parameters['dimensionXName'] = $widget['xKeysX_select'];
            }else{
                $parameters['dimensionXName'] = $widget['yAxisLabel'];
            }
            $parameters['factLabel'] = $widget['customLabels'][0];
            $parameters['factValue'] = $parameters['facts'][0];
        }
        return $parameters;
    }

    protected function getIndicatorParameters($indicatorId, &$parameters) {
        $indicator = $this->indicatorService->findById($indicatorId);
        $parameters["indicatorId"] = $indicatorId;
        $parameters["indicatorName"] = $indicator["name"];
    }

    protected function getDimensionParameters($widget, $dimension, &$parameters, $view) {
        if(isset($widget['allowMoreThanOneDimension']) && $widget['allowMoreThanOneDimension']){
            $dimensions = array();
            $dimensionProperties = array();

            if(isset($widget['dimensionProperty'])){
                $dimensionProperties = array($widget['dimensionProperty']);
                foreach ($dimensionProperties as $dimensionProperty) {
                    if(is_array($widget[$dimensionProperty])) {
                        $dimensions = array_merge($dimensions, $widget[$dimensionProperty]);
                    } else if($dimension && $widget[$dimension]) {
                        $dimensions[] = $widget[$dimensionProperty];
                    }
                }
            }

            $parameters["dimensions"] = $dimensions;
        }
        $dimensionProperty = isset($widget['dimensionProperty']) ? $widget['dimensionProperty'] : $view['dimensionProperty']['name'];
        if(!$dimension) {
            $dimension = isset($widget[$dimensionProperty]) ? $widget[$dimensionProperty] : null;
        }

        $dataSource = $this->dataSourceService->findByIndicator($widget['indicator']);
        $column = $this->widgetDataService->getDataSourceColumns($dataSource, $dimension, $view);
        if(!empty($column)) {
            $column = $column[0];
            $parameters["dimensionName"] = $column["name"];
            $parameters["dimensionLabel"] = $column["label"];
        }
    }

    protected function getFactsParameter($widget) {
        $facts = array();

        if($this->widgetFacts){
            $facts = $this->widgetFacts;
        } else if(isset($widget["factProperties"])) {
            foreach ($widget["factProperties"] as $fact) {
                if(is_array($widget[$fact])) {
                    $facts = array_merge($facts, $widget[$fact]);
                } else if($widget[$fact]) {
                    $facts[] = $widget[$fact];
                }
            }
        }

        return $facts;
    }

    protected function getOrderByParameters($view, $facts, $dimensions, &$parameters, $customOrderBy) {
        $indicatorId = $view['indicator'];
        $dataSource = $this->dataSourceService->findByIndicator($indicatorId);

        $factColumns = $this->widgetDataService->getDataSourceColumns($dataSource, $facts, $view);
        $dimensionColumns = $this->widgetDataService->getDataSourceColumns($dataSource, $dimensions, $view);

        $orderBy = $customOrderBy ?: $this->widgetDataService->getOrderBy($view, $factColumns, $dimensionColumns);

        if(!empty($orderBy)) {
            if(is_string(key($orderBy))){
                $keys = array_keys($orderBy);
                $orderByColumn = $this->widgetDataService->getDataSourceColumns($dataSource, $keys[0], $view);
                if(!empty($orderByColumn)) {
                    $orderByColumn = $orderByColumn[0];
                    $parameters["orderByName"] = $orderByColumn["name"];
                    $parameters["orderByLabel"] = $orderByColumn["label"];
                }
                $parameters["orderByDirection"] = $orderBy[$keys[0]];
            }
        }
    }

    protected function getPeriodParameters($widget, &$parameters, $customPeriod = null) {
        $period = null;

        if (!empty($customPeriod)){
            $period = $customPeriod;
        } else if (isset($widget['period']) && $this->checkInterval($widget['period'])){
            $period = $widget['period'];
        }
        if($period){
            if($this->hasMultiplePeriod($period)) {
                $periods = array();
                foreach ($period as $singlePeriod) {
                    $periods[] = $this->decodeFormattedPeriod($singlePeriod);
                }
                $period = $periods;
            } else {
                $period = $this->decodeFormattedPeriod($period);
                $parameters['period_start'] = $period["start"];
                $parameters['period_end'] = $period["end"];
            }
        }

        $parameters['period'] = $period;
    }

    protected function checkInterval($interval){
        return ($interval && (isset($interval[0]) && $interval[0]) || (isset($interval['start']) && $interval['start']));
    }

    public function decodeFormattedPeriod($period, $dateBase = '') {
        $formattedPeriod = array();
        if(is_string($period)) {
            list($formattedPeriod['start'], $formattedPeriod['end']) = Period::decodeFormatted($period,$dateBase);
            $periodObj = $this->getPeriodObjFromCode($period);
            if(isset($periodObj['labelOnView']) || isset($periodObj['name'])){
                $formattedPeriod['periodLabel'] = $periodObj['labelOnView'] ?: $periodObj['name'];
                if(!empty($periodObj['base'])){
                    $formattedPeriod['base'] = $periodObj['base'];
                }
            }
        } else {
            $formattedPeriod = $period;
        }
        return $formattedPeriod;
    }

    protected function getStructureParameters($indicatorId, &$parameters, $structureId = null, $userStructureHierarchy = null, $structureName = null, $hierarchyList = null) {

        $parameters['structureId'] = array();
        $parameters['structureName'] = array();
        if(isset($structureId) && $structureId != "" && isset($structureName) && $structureName != ""){
            $parameters['structureId'][] = $structureId;
            $parameters['structureName'][] = $structureName;
        }else{
            if(is_array($hierarchyList)){
                $structures = $hierarchyList[$indicatorId];
            } else {
                $structures = $this->hierarchyService->getIndicatorHierarchyByLoggedUser($indicatorId, null);
            }

            if(!empty($structures)) {
                $filteredStructures = $this->getStructuresByType($structures, $structures[0]['type']);
                foreach ($filteredStructures as $structure) {
                    $parameters['structureId'][] = $structure['_id'];
                    $parameters['structureName'][] = $structure['name'];
                }
            }
        }

    }

    public static function getStructuresByType($structures, $type) {
        $filteredStructures = array();
        foreach ($structures as $structure) {
            if($structure['type'] === $type) {
                $filteredStructures[] = $structure;
            }
        }
        return $filteredStructures;
    }

    public function getDefaultOrgViews(FilterCriteria $filterCriteria){
        $filterCriteria->setDataSourceName($this->dataSourceName);
        $defaultOrgViews = $this->indicatorService->getDataFromMultipleOrgs($filterCriteria);
        return $defaultOrgViews;
    }

    public function columnOrderBy($widgetColumns,$orderBy){
        if(is_array($orderBy)){
            foreach($orderBy as $singleOrderBy)
            {
                foreach($widgetColumns as $key => $column){
                    if($singleOrderBy["name"] ===  $column["name"]){
                        $widgetColumns[$key]["ordirnationPriority_inline"] = $singleOrderBy["priority"];
                        $widgetColumns[$key]["ordirnationPriority"] = $singleOrderBy["priority"];
                    }
                }
            }
        }
        return $widgetColumns;
    }

    protected function verifyZmartTotalized(&$widget){
        if(isset($widget['zmartTotalized']) && $widget['zmartTotalized'] && $widget['chartType'] === 'grid'){
            $structureField = array(
                "name" => "STRUCTURE",
                "grouped" => true
            );

            foreach ($widget['fields'] as &$field) {
                if(isset($field['isFact']) && $field['isFact']){
                    $field['expression'] = isset($field['function']) ? $field['function'] : "SUM";
                }
            }

            array_push($widget['fields'], $structureField);
        }
    }

    public function reserchForEach($request,$type)
    {
        $params = $request->getParameters();
        if (!array_key_exists("_id", $params)) return array();
        $id = $params["_id"];
        $filterData = new FilterCriteria("indicator");
        foreach ($params["in"] as $fieldName) {
            if ($params[$fieldName]) {
                $filterData->addCondition($fieldName, "=", $params[$fieldName]);
            }
        }
        $dataSet = $this->organizationManager->findBy($filterData)->getRows();

        if (isset($dataSet[0]["dataSourceColumns"])) {
            $result = array();
            foreach ($dataSet[0]["dataSourceColumns"] as $column) {
                if (isset($column[$type]) && $column[$type]) {
                    array_push($result, $column);
                }
            }
            return $result;
        } else {
            return array();
        }
    }

    private function fixBugReport($view){
        if(isset($view['fields']) && is_array($view['fields'])) {
            foreach ($view['fields'] as &$field) {
                if (isset($field['expression']) && $field['expression'] === "") {
                    unset($field['expression']);
                }
                if (isset($field['function']) && $field['function'] === "") {
                    unset($field['function']);
                }
            }
        }
        return $view;
    }

    private function setCrossTabProperty($view)
    {
        if ($view['template'] == "widget/crosstab.html") {
            $view['floatingControl']['xlsAction'] = array("runAt" => "server");
            $view['floatingControl']['pdfAction'] = array("runAt" => "server");
            $view['floatingControl']['csvAction'] = true;
            $view['fields'] = array();
            $view['template'] = "widget/grid.html";
            $view['hasPagination'] = false;
            $view['searchWithPagination'] = false;
            $view['isCrossTab'] = true;
            $view['events'][] = array(
                'name' => 'WidgetAfterLoadDataSource',
                'code' => 'ChartViewController.setFieldsInCrossTab(args.owner);'
            );
        }
        return $view;
    }

}
